@@ -172,7 +172,6 @@ namespace Oxygen
         //@{
         bool eventFilterComboBoxContainer( QWidget*, QEvent* );
         bool eventFilterDockWidget( QDockWidget*, QEvent* );
-        bool eventFilterGeometryTip( QWidget*, QEvent* );
         bool eventFilterMdiSubWindow( QMdiSubWindow*, QEvent* );
         bool eventFilterQ3ListView( QWidget*, QEvent* );
         bool eventFilterScrollBar( QWidget*, QEvent* );
@@ -596,7 +595,7 @@ namespace Oxygen
 
         //! true if widget is child of KTextEdit
         bool isKTextEditFrame( const QWidget* widget ) const
-        { return ( widget && widget->parentWidget() && qobject_cast<const QFrame*>( widget ) && widget->parentWidget()->inherits( "KTextEditor::View" ) ); }
+        { return ( widget && widget->parentWidget() && widget->parentWidget()->inherits( "KTextEditor::View" ) ); }
 
         //! adjust rect based on provided margins
         QRect insideMargin( const QRect& r, int main, int left = 0, int top = 0, int right = 0, int bottom = 0 ) const
@@ -903,6 +902,10 @@ namespace Oxygen
         int _subElementCounter;
         QHash<QString, int> _styleElements;
 
+        //! use Argb Drag and Drop Window
+        QStyle::StyleHint SH_ArgbDndWindow;
+
+        //! styled painting for KCapacityBar
         QStyle::ControlElement CE_CapacityBar;
 
         //@}
