@@ -49,7 +49,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "effects.h"
 #include "overlaywindow.h"
 #include "scene.h"
-#include "scene_basic.h"
 #include "scene_xrender.h"
 #include "scene_opengl.h"
 #include "shadow.h"
@@ -58,6 +57,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <stdio.h>
 
+#include <QtCore/QtConcurrentRun>
+#include <QtCore/QFutureWatcher>
 #include <QMenu>
 #include <QTimerEvent>
 #include <kaction.h>
@@ -91,25 +92,39 @@ void Workspace::setupCompositing()
         return;
     }
 
-    if (!options->compositingInitialized)
+    if (!options->isCompositingInitialized()) {
+#ifndef KWIN_HAVE_OPENGLES
+        // options->reloadCompositingSettings(true) initializes the CompositingPrefs which calls an
+        // external program in turn
+        // run this in an external thread to make startup faster.
+        QFutureWatcher<void> *compositingPrefsFuture = new QFutureWatcher<void>();
+        connect(compositingPrefsFuture, SIGNAL(finished()), this, SLOT(slotCompositingOptionsInitialized()));
+        connect(compositingPrefsFuture, SIGNAL(finished()), compositingPrefsFuture, SLOT(deleteLater()));
+        compositingPrefsFuture->setFuture(QtConcurrent::run(options, &Options::reloadCompositingSettings, true));
+#else
+        // OpenGL ES does not call the external program, so no need to create a thread
         options->reloadCompositingSettings(true);
+        slotCompositingOptionsInitialized();
+#endif
+    } else {
+        slotCompositingOptionsInitialized();
+    }
+}
 
+void Workspace::slotCompositingOptionsInitialized()
+{
     char selection_name[ 100 ];
     sprintf(selection_name, "_NET_WM_CM_S%d", DefaultScreen(display()));
     cm_selection = new KSelectionOwner(selection_name);
     connect(cm_selection, SIGNAL(lostOwnership()), SLOT(lostCMSelection()));
     cm_selection->claim(true);   // force claiming
 
-    switch(options->compositingMode) {
-        /*case 'B':
-            kDebug( 1212 ) << "X compositing";
-            scene = new SceneBasic( this );
-          break; // don't fall through (this is a testing one) */
+    switch(options->compositingMode()) {
     case OpenGLCompositing: {
         kDebug(1212) << "Initializing OpenGL compositing";
 
         // Some broken drivers crash on glXQuery() so to prevent constant KWin crashes:
-        KSharedConfigPtr unsafeConfigPtr(KSharedConfig::openConfig("kwinrc"));
+        KSharedConfigPtr unsafeConfigPtr = KGlobal::config();
         KConfigGroup unsafeConfig(unsafeConfigPtr, "Compositing");
         if (unsafeConfig.readEntry("OpenGLIsUnsafe", false))
             kWarning(1212) << "KWin has detected that your OpenGL library is unsafe to use";
@@ -160,26 +175,28 @@ void Workspace::setupCompositing()
         return;
     }
     xrrRefreshRate = KWin::currentRefreshRate();
-    fpsInterval = (options->maxFpsInterval << 10);
+    fpsInterval = (options->maxFpsInterval() << 10);
     if (scene->waitSyncAvailable()) {  // if we do vsync, set the fps to the next multiple of the vblank rate
         vBlankInterval = (1000 << 10) / xrrRefreshRate;
-        fpsInterval -= (fpsInterval % vBlankInterval);
-        fpsInterval = qMax(fpsInterval, vBlankInterval);
+        fpsInterval = qMax((fpsInterval / vBlankInterval) * vBlankInterval, vBlankInterval);
     } else
         vBlankInterval = 1 << 10; // no sync - DO NOT set "0", would cause div-by-zero segfaults.
-    vBlankPadding = 3; // vblank rounding errors... :-(
-    nextPaintReference.start();
+    m_timeSinceLastVBlank = fpsInterval - 1; // means "start now" - we dont't have even a slight idea when the first vsync will occur
     checkCompositeTimer();
     XCompositeRedirectSubwindows(display(), rootWindow(), CompositeRedirectManual);
     new EffectsHandlerImpl(scene->compositingType());   // sets also the 'effects' pointer
     addRepaintFull();
     foreach (Client * c, clients)
-    c->setupCompositing();
+        c->setupCompositing();
     foreach (Client * c, desktops)
-    c->setupCompositing();
+        c->setupCompositing();
     foreach (Unmanaged * c, unmanaged)
-    c->setupCompositing();
+        c->setupCompositing();
     discardPopup(); // force re-creation of the Alt+F3 popup (opacity option)
+
+    // render at least once
+    compositeTimer.stop();
+    performCompositing();
 }
 
 void Workspace::finishCompositing()
@@ -232,7 +249,7 @@ void Workspace::finishCompositing()
 void Workspace::fallbackToXRenderCompositing()
 {
     finishCompositing();
-    KConfigGroup config(KSharedConfig::openConfig("kwinrc"), "Compositing");
+    KConfigGroup config(KGlobal::config(), "Compositing");
     config.writeEntry("Backend", "XRender");
     config.writeEntry("GraphicsSystem", "native");
     config.sync();
@@ -240,7 +257,7 @@ void Workspace::fallbackToXRenderCompositing()
         restartKWin("automatic graphicssystem change for XRender backend");
         return;
     } else {
-        options->compositingMode = XRenderCompositing;
+        options->setCompositingMode(XRenderCompositing);
         setupCompositing();
     }
 }
@@ -375,12 +392,15 @@ void Workspace::timerEvent(QTimerEvent *te)
     } else
         QObject::timerEvent(te);
 }
-
+static bool s_pending = false;
+QElapsedTimer profiler;
 void Workspace::performCompositing()
 {
-    if (((repaints_region.isEmpty() && !windowRepaintsPending())  // no damage
-            || !scene->overlayWindow()->isVisible())) { // nothing is visible anyway
-        vBlankPadding += 3;
+    if (!scene->overlayWindow()->isVisible())
+        return; // nothing is visible anyway
+
+    bool pending = !repaints_region.isEmpty() || windowRepaintsPending();
+    if (!(pending || s_pending)) {
         scene->idle();
         // Note: It would seem here we should undo suspended unredirect, but when scenes need
         // it for some reason, e.g. transformations or translucency, the next pass that does not
@@ -388,6 +408,8 @@ void Workspace::performCompositing()
         // Otherwise the window would not be painted normally anyway.
         return;
     }
+    profiler.start();
+    s_pending = pending;
     // create a list of all windows in the stacking order
     ToplevelList windows = xStackingOrder();
     foreach (EffectWindow *c, static_cast< EffectsHandlerImpl* >(effects)->elevatedWindows()) {
@@ -407,17 +429,8 @@ void Workspace::performCompositing()
     QRegion repaints = repaints_region;
     // clear all repaints, so that post-pass can add repaints for the next repaint
     repaints_region = QRegion();
-    if (scene->waitSyncAvailable()) {
-        // vsync: paint the scene, than rebase the timer and use the duration for next timeout estimation
-        scene->paint(repaints, windows);
-        nextPaintReference.start();
-    } else {
-        // no vsyc -> inversion: reset the timer, then paint the scene, this way we can provide a constant framerate
-        nextPaintReference.start();
-        scene->paint(repaints, windows);
-    }
-    // reset the roundin error corrective... :-(
-    vBlankPadding = 3;
+
+    m_timeSinceLastVBlank = scene->paint(repaints, windows);
     // Trigger at least one more pass even if there would be nothing to paint, so that scene->idle()
     // is called the next time. If there would be nothing pending, it will not restart the timer and
     // checkCompositeTime() would restart it again somewhen later, called from functions that
@@ -452,19 +465,38 @@ void Workspace::setCompositeTimer()
     if (!compositing())  // should not really happen, but there may be e.g. some damage events still pending
         return;
 
-    // interval - "time since last paint completion" - "time we need to paint"
-    uint passed = nextPaintReference.elapsed() << 10;
-    uint delay = fpsInterval;
-    if (scene->waitSyncAvailable()) {
-        if (passed > fpsInterval) {
-            delay = vBlankInterval;
-            passed %= vBlankInterval;
-        }
-        delay -= ((passed + ((scene->estimatedRenderTime() + vBlankPadding) << 10)) % vBlankInterval);
-    } else
-        delay = qBound(0, int(delay - passed), 250 << 10);
+    uint padding = m_timeSinceLastVBlank << 10;
 
-    compositeTimer.start(delay >> 10, this);
+    if (scene->waitSyncAvailable()) {
+
+        // TODO: make vBlankTime dynamic?!
+        // It's required because glXWaitVideoSync will *likely* block a full frame if one enters
+        // a retrace pass which can last a variable amount of time, depending on the actual screen
+        // Now, my ooold 19" CRT can do such retrace so that 2ms are entirely sufficient,
+        // while another ooold 15" TFT requires about 6ms
+
+        if (padding > fpsInterval) {
+            // we're at low repaints or spent more time in painting than the user wanted to wait for that frame
+            padding = vBlankInterval - (padding%vBlankInterval); // -> align to next vblank
+        } else {  // -> align to the next maxFps tick
+            padding = ((vBlankInterval - padding%vBlankInterval) + (fpsInterval/vBlankInterval-1)*vBlankInterval);
+            //               "remaining time of the first vsync" + "time for the other vsyncs of the frame"
+        }
+
+        if (padding < options->vBlankTime()) { // we'll likely miss this frame
+            m_nextFrameDelay = (padding + vBlankInterval) >> 10;
+            padding = (padding + vBlankInterval - options->vBlankTime()) >> 10; // so we add one
+//             qDebug() << "WE LOST A FRAME";
+        } else {
+            m_nextFrameDelay = padding >> 10;
+            padding = (padding - options->vBlankTime()) >> 10;
+        }
+    }
+    else // w/o vsync we just jump to the next demanded tick
+        // the "1" will ensure we don't block out the eventloop - the system's just not faster
+        // "0" would be sufficient, but the compositor isn't the WMs only task
+        m_nextFrameDelay = padding = (padding > (int)fpsInterval) ? 1 : ((fpsInterval - padding) >> 10);
+    compositeTimer.start(qMin(padding, 250u), this); // force 4fps minimum
 }
 
 void Workspace::startMousePolling()
@@ -485,7 +517,7 @@ bool Workspace::compositingActive()
 // force is needed when the list of windows changes (e.g. a window goes away)
 void Workspace::checkUnredirect(bool force)
 {
-    if (!compositing() || scene->overlayWindow()->window() == None || !options->unredirectFullscreen)
+    if (!compositing() || scene->overlayWindow()->window() == None || !options->isUnredirectFullscreen())
         return;
     if (force)
         forceUnredirectCheck = true;
@@ -495,7 +527,7 @@ void Workspace::checkUnredirect(bool force)
 
 void Workspace::delayedCheckUnredirect()
 {
-    if (!compositing() || scene->overlayWindow()->window() == None || !options->unredirectFullscreen)
+    if (!compositing() || scene->overlayWindow()->window() == None || !options->isUnredirectFullscreen())
         return;
     ToplevelList list;
     bool changed = forceUnredirectCheck;
@@ -521,24 +553,62 @@ void Workspace::delayedCheckUnredirect()
     scene->overlayWindow()->setShape(reg);
 }
 
+
+bool Workspace::compositingPossible() const
+{
+    return CompositingPrefs::compositingPossible();
+}
+
+QString Workspace::compositingNotPossibleReason() const
+{
+    return CompositingPrefs::compositingNotPossibleReason();
+}
+
+bool Workspace::openGLIsBroken() const
+{
+    return CompositingPrefs::openGlIsBroken();
+}
+
+QString Workspace::compositingType()
+{
+    // the returned strings are considered as identifiers and may not be translated
+    if (!effects) {
+        return "none";
+    }
+    if (effects->compositingType() == XRenderCompositing) {
+        return "xrender";
+    } else if (effects->compositingType() == OpenGLCompositing) {
+#ifdef KWIN_HAVE_OPENGLES
+        return "gles";
+#else
+        if (ShaderManager::instance()->isValid()) {
+            return "gl2";
+        } else {
+            return "gl1";
+        }
+#endif
+    }
+    return "none";
+}
+
 //****************************************
 // Toplevel
 //****************************************
 
-void Toplevel::setupCompositing()
+bool Toplevel::setupCompositing()
 {
     if (!compositing())
-        return;
+        return false;
     damageRatio = 0.0;
     if (damage_handle != None)
-        return;
+        return false;
     damage_handle = XDamageCreate(display(), frameId(), XDamageReportRawRectangles);
     damage_region = QRegion(0, 0, width(), height());
-    effect_window = new EffectWindowImpl();
-    effect_window->setWindow(this);
+    effect_window = new EffectWindowImpl(this);
     unredirect = false;
     workspace()->checkUnredirect(true);
     scene->windowAdded(this);
+    return true;
 }
 
 void Toplevel::finishCompositing()
@@ -620,7 +690,7 @@ void Toplevel::damageNotifyEvent(XDamageNotifyEvent* e)
 
     const float area = rect().width()*rect().height();
     damageRects.reserve(16);
-    damageRects.clear();
+    damageRects.erase(damageRects.begin(), damageRects.end());
     damageRects << QRect(e->area.x, e->area.y, e->area.width, e->area.height);
 
     // we can not easily say anything about the overall ratio since the new rects may intersect the present
@@ -663,10 +733,12 @@ void Client::damageNotifyEvent(XDamageNotifyEvent* e)
 #ifdef HAVE_XSYNC
     if (syncRequest.isPending && isResize())
         return;
-    if (syncRequest.counter == None)   // cannot detect complete redraw, consider done now
-        ready_for_painting = true;
+    if (!ready_for_painting) { // avoid "setReadyForPainting()" function calling overhead
+        if (syncRequest.counter == None)   // cannot detect complete redraw, consider done now
+            setReadyForPainting();
+    }
 #else
-    ready_for_painting = true;
+        setReadyForPainting();
 #endif
 
     Toplevel::damageNotifyEvent(e);
@@ -739,24 +811,48 @@ void Toplevel::resetDamage(const QRect& r)
 
 void Toplevel::addRepaint(const QRect& r)
 {
-    addRepaint(r.x(), r.y(), r.width(), r.height());
-}
-
-void Toplevel::addRepaint(int x, int y, int w, int h)
-{
-    if (!compositing())
+    if (!compositing()) {
         return;
-    QRect r(x, y, w, h);
-    r &= rect();
+    }
     repaints_region += r;
     workspace()->checkCompositeTimer();
 }
 
+void Toplevel::addRepaint(int x, int y, int w, int h)
+{
+    QRect r(x, y, w, h);
+    addRepaint(r);
+}
+
 void Toplevel::addRepaint(const QRegion& r)
+{
+    if (!compositing()) {
+        return;
+    }
+    repaints_region += r;
+    workspace()->checkCompositeTimer();
+}
+
+void Toplevel::addLayerRepaint(const QRect& r)
+{
+    if (!compositing()) {
+        return;
+    }
+    layer_repaints_region += r;
+    workspace()->checkCompositeTimer();
+}
+
+void Toplevel::addLayerRepaint(int x, int y, int w, int h)
+{
+    QRect r(x, y, w, h);
+    addLayerRepaint(r);
+}
+
+void Toplevel::addLayerRepaint(const QRegion& r)
 {
     if (!compositing())
         return;
-    repaints_region += r;
+    layer_repaints_region += r;
     workspace()->checkCompositeTimer();
 }
 
@@ -766,9 +862,10 @@ void Toplevel::addRepaintFull()
     workspace()->checkCompositeTimer();
 }
 
-void Toplevel::resetRepaints(const QRect& r)
+void Toplevel::resetRepaints()
 {
-    repaints_region -= r;
+    repaints_region = QRegion();
+    layer_repaints_region = QRegion();
 }
 
 void Toplevel::addWorkspaceRepaint(int x, int y, int w, int h)
@@ -815,19 +912,27 @@ void Toplevel::suspendUnredirect(bool suspend)
 // Client
 //****************************************
 
-void Client::setupCompositing()
+bool Client::setupCompositing()
 {
-    Toplevel::setupCompositing();
+    if (!Toplevel::setupCompositing()){
+        return false;
+    }
     updateVisibility(); // for internalKeep()
-    updateDecoration(true, true);
-    move(calculateGravitation(true)); // we just polluted the gravity because the window likely has no decoration yet
+    if (isManaged()) {
+        // only create the decoration when a client is managed
+        updateDecoration(true, true);
+    }
+    return true;
 }
 
 void Client::finishCompositing()
 {
     Toplevel::finishCompositing();
     updateVisibility();
-    updateDecoration(true, true);
+    if (!deleting) {
+        // only recreate the decoration if we are not shutting down completely
+        updateDecoration(true, true);
+    }
     // for safety in case KWin is just resizing the window
     s_haveResizeEffect = false;
 }
