@@ -30,11 +30,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef KWIN_BUILD_TABBOX
 #include "tabbox.h"
 #endif
+#include "scripting/scriptedeffect.h"
 #include "thumbnailitem.h"
 #include "workspace.h"
 #include "kwinglutils.h"
 
 #include <QFile>
+#include <QtCore/QFutureWatcher>
+#include <QtCore/QtConcurrentRun>
 
 #include "kdebug.h"
 #include "klibrary.h"
@@ -103,7 +106,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     m_currentBuildQuadsIterator = m_activeEffects.end();
 
     Workspace *ws = Workspace::self();
-    connect(ws, SIGNAL(currentDesktopChanged(int)), this, SLOT(slotDesktopChanged(int)));
+    connect(ws, SIGNAL(currentDesktopChanged(int, KWin::Client*)), SLOT(slotDesktopChanged(int, KWin::Client*)));
     connect(ws, SIGNAL(clientAdded(KWin::Client*)), this, SLOT(slotClientAdded(KWin::Client*)));
     connect(ws, SIGNAL(unmanagedAdded(KWin::Unmanaged*)), this, SLOT(slotUnmanagedAdded(KWin::Unmanaged*)));
     connect(ws, SIGNAL(clientActivated(KWin::Client*)), this, SLOT(slotClientActivated(KWin::Client*)));
@@ -149,6 +152,7 @@ void EffectsHandlerImpl::setupClientConnections(Client* c)
     connect(c, SIGNAL(clientMinimized(KWin::Client*,bool)), this, SLOT(slotClientMinimized(KWin::Client*,bool)));
     connect(c, SIGNAL(clientUnminimized(KWin::Client*,bool)), this, SLOT(slotClientUnminimized(KWin::Client*,bool)));
     connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), this, SLOT(slotGeometryShapeChanged(KWin::Toplevel*,QRect)));
+    connect(c, SIGNAL(paddingChanged(KWin::Toplevel*,QRect)), this, SLOT(slotPaddingChanged(KWin::Toplevel*,QRect)));
     connect(c, SIGNAL(damaged(KWin::Toplevel*,QRect)), this, SLOT(slotWindowDamaged(KWin::Toplevel*,QRect)));
     connect(c, SIGNAL(propertyNotify(KWin::Toplevel*,long)), this, SLOT(slotPropertyNotify(KWin::Toplevel*,long)));
 }
@@ -158,18 +162,32 @@ void EffectsHandlerImpl::setupUnmanagedConnections(Unmanaged* u)
     connect(u, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), this, SLOT(slotWindowClosed(KWin::Toplevel*)));
     connect(u, SIGNAL(opacityChanged(KWin::Toplevel*,qreal)), this, SLOT(slotOpacityChanged(KWin::Toplevel*,qreal)));
     connect(u, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), this, SLOT(slotGeometryShapeChanged(KWin::Toplevel*,QRect)));
+    connect(u, SIGNAL(paddingChanged(KWin::Toplevel*,QRect)), this, SLOT(slotPaddingChanged(KWin::Toplevel*,QRect)));
     connect(u, SIGNAL(damaged(KWin::Toplevel*,QRect)), this, SLOT(slotWindowDamaged(KWin::Toplevel*,QRect)));
     connect(u, SIGNAL(propertyNotify(KWin::Toplevel*,long)), this, SLOT(slotPropertyNotify(KWin::Toplevel*,long)));
 }
 
 void EffectsHandlerImpl::reconfigure()
 {
-    KSharedConfig::Ptr _config = KGlobal::config();
-    KConfigGroup conf(_config, "Plugins");
+    // perform querying for the services in a thread
+    QFutureWatcher<KService::List> *watcher = new QFutureWatcher<KService::List>(this);
+    connect(watcher, SIGNAL(finished()), this, SLOT(slotEffectsQueried()));
+    watcher->setFuture(QtConcurrent::run(KServiceTypeTrader::self(), &KServiceTypeTrader::query, QString("KWin/Effect"), QString()));
+}
 
-    KService::List offers = KServiceTypeTrader::self()->query("KWin/Effect");
+void EffectsHandlerImpl::slotEffectsQueried()
+{
+    QFutureWatcher<KService::List> *watcher = dynamic_cast< QFutureWatcher<KService::List>* >(sender());
+    if (!watcher) {
+        // slot invoked not from a FutureWatcher
+        return;
+    }
+
+    KService::List offers = watcher->result();
     QStringList effectsToBeLoaded;
     QStringList checkDefault;
+    KSharedConfig::Ptr _config = KGlobal::config();
+    KConfigGroup conf(_config, "Plugins");
 
     // First unload necessary effects
     foreach (const KService::Ptr & service, offers) {
@@ -201,6 +219,7 @@ void EffectsHandlerImpl::reconfigure()
         if (!newLoaded.contains(ep.first))    // don't reconfigure newly loaded effects
             ep.second->reconfigure(Effect::ReconfigureAll);
     }
+    watcher->deleteLater();
 }
 
 // the idea is that effects call this function again which calls the next one
@@ -269,12 +288,12 @@ void EffectsHandlerImpl::postPaintWindow(EffectWindow* w)
     // no special final code
 }
 
-bool EffectsHandlerImpl::provides(Effect::Feature ef)
+Effect *EffectsHandlerImpl::provides(Effect::Feature ef)
 {
     for (int i = 0; i < loaded_effects.size(); ++i)
         if (loaded_effects.at(i).second->provides(ef))
-            return true;
-    return false;
+            return loaded_effects.at(i).second;
+    return NULL;
 }
 
 void EffectsHandlerImpl::drawWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data)
@@ -288,10 +307,17 @@ void EffectsHandlerImpl::drawWindow(EffectWindow* w, int mask, QRegion region, W
 
 void EffectsHandlerImpl::buildQuads(EffectWindow* w, WindowQuadList& quadList)
 {
+    static bool initIterator = true;
+    if (initIterator) {
+        m_currentBuildQuadsIterator = m_activeEffects.begin();
+        initIterator = false;
+    }
     if (m_currentBuildQuadsIterator != m_activeEffects.end()) {
         (*m_currentBuildQuadsIterator++)->buildQuads(w, quadList);
         --m_currentBuildQuadsIterator;
     }
+    if (m_currentBuildQuadsIterator == m_activeEffects.begin())
+        initIterator = true;
 }
 
 bool EffectsHandlerImpl::hasDecorationShadows() const
@@ -322,7 +348,6 @@ void EffectsHandlerImpl::startPaint()
     m_currentPaintWindowIterator = m_activeEffects.begin();
     m_currentPaintScreenIterator = m_activeEffects.begin();
     m_currentPaintEffectFrameIterator = m_activeEffects.begin();
-    m_currentBuildQuadsIterator = m_activeEffects.begin();
 }
 
 void EffectsHandlerImpl::slotClientMaximized(KWin::Client *c, KDecorationDefines::MaximizeMode maxMode)
@@ -373,16 +398,27 @@ void EffectsHandlerImpl::slotOpacityChanged(Toplevel *t, qreal oldOpacity)
 
 void EffectsHandlerImpl::slotClientAdded(Client *c)
 {
-    setupClientConnections(c);
-    emit windowAdded(c->effectWindow());
+    if (c->readyForPainting())
+        slotClientShown(c);
+    else
+        connect(c, SIGNAL(windowShown(KWin::Toplevel*)), SLOT(slotClientShown(KWin::Toplevel*)));
 }
 
 void EffectsHandlerImpl::slotUnmanagedAdded(Unmanaged *u)
-{
+{   // regardless, unmanaged windows are -yet?- not synced anyway
     setupUnmanagedConnections(u);
     emit windowAdded(u->effectWindow());
 }
 
+void EffectsHandlerImpl::slotClientShown(KWin::Toplevel *t)
+{
+    Q_ASSERT(dynamic_cast<Client*>(t));
+    Client *c = static_cast<Client*>(t);
+    setupClientConnections(c);
+    if (!c->tabGroup()) // the "window" has already been there
+        emit windowAdded(c->effectWindow());
+}
+
 void EffectsHandlerImpl::slotDeletedRemoved(KWin::Deleted *d)
 {
     emit windowDeleted(d->effectWindow());
@@ -415,25 +451,27 @@ void EffectsHandlerImpl::slotClientUnminimized(Client* c, bool animate)
     }
 }
 
-void EffectsHandlerImpl::slotClientGroupItemSwitched(EffectWindow* from, EffectWindow* to)
+void EffectsHandlerImpl::slotCurrentTabAboutToChange(EffectWindow *from, EffectWindow *to)
 {
-    emit clientGroupItemSwitched(from, to);
+    emit currentTabAboutToChange(from, to);
 }
 
-void EffectsHandlerImpl::slotClientGroupItemAdded(EffectWindow* from, EffectWindow* to)
+void EffectsHandlerImpl::slotTabAdded(EffectWindow* w, EffectWindow* to)
 {
-    emit clientGroupItemAdded(from, to);
+    emit tabAdded(w, to);
 }
 
-void EffectsHandlerImpl::slotClientGroupItemRemoved(EffectWindow* c, EffectWindow* group)
+void EffectsHandlerImpl::slotTabRemoved(EffectWindow *w, EffectWindow* leaderOfFormerGroup)
 {
-    emit clientGroupItemRemoved(c, group);
+    emit tabRemoved(w, leaderOfFormerGroup);
 }
 
-void EffectsHandlerImpl::slotDesktopChanged(int old)
+void EffectsHandlerImpl::slotDesktopChanged(int old, Client *c)
 {
     const int newDesktop = Workspace::self()->currentDesktop();
     if (old != 0 && newDesktop != old) {
+        emit desktopChanged(old, newDesktop, c ? c->effectWindow() : 0);
+        // TODO: remove in 4.10
         emit desktopChanged(old, newDesktop);
     }
 }
@@ -456,6 +494,15 @@ void EffectsHandlerImpl::slotGeometryShapeChanged(Toplevel* t, const QRect& old)
     emit windowGeometryShapeChanged(t->effectWindow(), old);
 }
 
+void EffectsHandlerImpl::slotPaddingChanged(Toplevel* t, const QRect& old)
+{
+    // during late cleanup effectWindow() may be already NULL
+    // in some functions that may still call this
+    if (t == NULL || t->effectWindow() == NULL)
+        return;
+    emit windowPaddingChanged(t->effectWindow(), old);
+}
+
 void EffectsHandlerImpl::setActiveFullScreenEffect(Effect* e)
 {
     fullscreen_effect = e;
@@ -532,6 +579,13 @@ bool EffectsHandlerImpl::hasKeyboardGrab() const
     return keyboard_grab_effect != NULL;
 }
 
+void EffectsHandlerImpl::desktopResized(const QSize &size)
+{
+    scene->screenGeometryChanged(size);
+    emit screenGeometryChanged(size);
+    Workspace::self()->addRepaintFull();
+}
+
 void EffectsHandlerImpl::slotPropertyNotify(Toplevel* t, long int atom)
 {
     if (!registered_atoms.contains(atom))
@@ -700,7 +754,7 @@ QString EffectsHandlerImpl::desktopName(int desktop) const
 
 bool EffectsHandlerImpl::optionRollOverDesktops() const
 {
-    return options->rollOverDesktops;
+    return options->isRollOverDesktops();
 }
 
 double EffectsHandlerImpl::animationTimeFactor() const
@@ -1029,10 +1083,10 @@ void EffectsHandlerImpl::unreserveElectricBorder(ElectricBorder border)
 #endif
 }
 
-void EffectsHandlerImpl::reserveElectricBorderSwitching(bool reserve)
+void EffectsHandlerImpl::reserveElectricBorderSwitching(bool reserve, Qt::Orientations o)
 {
 #ifdef KWIN_BUILD_SCREENEDGES
-    Workspace::self()->screenEdge()->reserveDesktopSwitching(reserve);
+    Workspace::self()->screenEdge()->reserveDesktopSwitching(reserve, o);
 #else
     Q_UNUSED(reserve)
 #endif
@@ -1055,6 +1109,7 @@ KLibrary* EffectsHandlerImpl::findEffectLibrary(KService* service)
         libname.replace("kwin4_effect_", "kwin4_effect_gles_");
     }
 #endif
+    libname.replace("kwin", KWIN_NAME);
     KLibrary* library = new KLibrary(libname);
     if (!library) {
         kError(1212) << "couldn't open library for effect '" <<
@@ -1121,6 +1176,11 @@ bool EffectsHandlerImpl::loadEffect(const QString& name, bool checkDefault)
     }
     KService::Ptr service = offers.first();
 
+    if (service->property("X-Plasma-API").toString() == "javascript") {
+        // this is a scripted effect - use different loader
+        return loadScriptedEffect(name, service.data());
+    }
+
     KLibrary* library = findEffectLibrary(service.data());
     if (!library) {
         return false;
@@ -1204,6 +1264,29 @@ bool EffectsHandlerImpl::loadEffect(const QString& name, bool checkDefault)
     return true;
 }
 
+bool EffectsHandlerImpl::loadScriptedEffect(const QString& name, KService *service)
+{
+    const KDesktopFile df("services", service->entryPath());
+    const QString scriptName = df.desktopGroup().readEntry<QString>("X-Plasma-MainScript", "");
+    if (scriptName.isEmpty()) {
+        kDebug(1212) << "X-Plasma-MainScript not set";
+        return false;
+    }
+    const QString scriptFile = KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + "/effects/" + name + "/contents/" + scriptName);
+    if (scriptFile.isNull()) {
+        kDebug(1212) << "Could not locate the effect script";
+        return false;
+    }
+    ScriptedEffect *effect = ScriptedEffect::create(name, scriptFile);
+    if (!effect) {
+        kDebug(1212) << "Could not initialize scripted effect: " << name;
+        return false;
+    }
+    effect_order.insert(service->property("X-KDE-Ordering").toInt(), EffectPair(name, effect));
+    effectsChanged();
+    return true;
+}
+
 void EffectsHandlerImpl::unloadEffect(const QString& name)
 {
     Workspace::self()->addRepaintFull();
@@ -1217,7 +1300,9 @@ void EffectsHandlerImpl::unloadEffect(const QString& name)
             delete it.value().second;
             effect_order.erase(it);
             effectsChanged();
-            effect_libraries[ name ]->unload();
+            if (effect_libraries.contains(name)) {
+                effect_libraries[ name ]->unload();
+            }
             return;
         }
     }
@@ -1243,6 +1328,21 @@ bool EffectsHandlerImpl::isEffectLoaded(const QString& name)
     return false;
 }
 
+void EffectsHandlerImpl::reloadEffect(Effect *effect)
+{
+    QString effectName;
+    for (QVector< EffectPair >::iterator it = loaded_effects.begin(); it != loaded_effects.end(); ++it) {
+        if ((*it).second == effect) {
+            effectName = (*it).first;
+            break;
+        }
+    }
+    if (!effectName.isNull()) {
+        unloadEffect(effectName);
+        loadEffect(effectName);
+    }
+}
+
 void EffectsHandlerImpl::effectsChanged()
 {
     loaded_effects.clear();
@@ -1294,10 +1394,9 @@ void EffectsHandlerImpl::slotHideOutline()
 // EffectWindowImpl
 //****************************************
 
-EffectWindowImpl::EffectWindowImpl()
-    : QObject(NULL)
-    , EffectWindow()
-    , toplevel(NULL)
+EffectWindowImpl::EffectWindowImpl(Toplevel *toplevel)
+    : EffectWindow(toplevel)
+    , toplevel(toplevel)
     , sw(NULL)
 {
 }
@@ -1326,66 +1425,6 @@ void EffectWindowImpl::disablePainting(int reason)
     sceneWindow()->disablePainting(reason);
 }
 
-void EffectWindowImpl::addRepaint(const QRect& r)
-{
-    toplevel->addRepaint(r);
-}
-
-void EffectWindowImpl::addRepaint(int x, int y, int w, int h)
-{
-    toplevel->addRepaint(x, y, w, h);
-}
-
-void EffectWindowImpl::addRepaintFull()
-{
-    toplevel->addRepaintFull();
-}
-
-int EffectWindowImpl::desktop() const
-{
-    return toplevel->desktop();
-}
-
-bool EffectWindowImpl::isOnActivity(QString activity) const
-{
-    return toplevel->isOnActivity(activity);
-}
-
-bool EffectWindowImpl::isOnAllActivities() const
-{
-    return toplevel->isOnAllActivities();
-}
-
-bool EffectWindowImpl::isOnAllDesktops() const
-{
-    return desktop() == NET::OnAllDesktops;
-}
-
-QString EffectWindowImpl::caption() const
-{
-    if (Client* c = dynamic_cast<Client*>(toplevel))
-        return c->caption();
-    else
-        return "";
-}
-
-QString EffectWindowImpl::windowClass() const
-{
-    return toplevel->resourceName() + ' ' + toplevel->resourceClass();
-}
-
-QString EffectWindowImpl::windowRole() const
-{
-    return toplevel->windowRole();
-}
-
-QPixmap EffectWindowImpl::icon() const
-{
-    if (Client* c = dynamic_cast<Client*>(toplevel))
-        return c->icon();
-    return QPixmap(); // TODO
-}
-
 const EffectWindowGroup* EffectWindowImpl::group() const
 {
     if (Client* c = dynamic_cast< Client* >(toplevel))
@@ -1393,30 +1432,6 @@ const EffectWindowGroup* EffectWindowImpl::group() const
     return NULL; // TODO
 }
 
-bool EffectWindowImpl::isMinimized() const
-{
-    if (Client* c = dynamic_cast<Client*>(toplevel))
-        return c->isMinimized();
-    else
-        return false;
-}
-
-double EffectWindowImpl::opacity() const
-{
-    return toplevel->opacity();
-}
-
-bool EffectWindowImpl::hasAlpha() const
-{
-    return toplevel->hasAlpha();
-}
-
-
-bool EffectWindowImpl::isDeleted() const
-{
-    return (dynamic_cast<Deleted*>(toplevel) != 0);
-}
-
 void EffectWindowImpl::refWindow()
 {
     if (Deleted* d = dynamic_cast< Deleted* >(toplevel))
@@ -1434,6 +1449,7 @@ void EffectWindowImpl::unrefWindow()
 void EffectWindowImpl::setWindow(Toplevel* w)
 {
     toplevel = w;
+    setParent(w);
 }
 
 void EffectWindowImpl::setSceneWindow(Scene::Window* w)
@@ -1441,72 +1457,11 @@ void EffectWindowImpl::setSceneWindow(Scene::Window* w)
     sw = w;
 }
 
-int EffectWindowImpl::x() const
-{
-    return toplevel->x();
-}
-
-int EffectWindowImpl::y() const
-{
-    return toplevel->y();
-}
-
-int EffectWindowImpl::width() const
-{
-    return toplevel->width();
-}
-
-int EffectWindowImpl::height() const
-{
-    return toplevel->height();
-}
-
-QSize EffectWindowImpl::basicUnit() const
-{
-    Client *client = dynamic_cast<Client*>(toplevel);
-    return client ? client->basicUnit() : QSize(1,1);
-}
-
-QRect EffectWindowImpl::geometry() const
-{
-    return toplevel->geometry();
-}
-
 QRegion EffectWindowImpl::shape() const
 {
     return sw ? sw->shape() : geometry();
 }
 
-int EffectWindowImpl::screen() const
-{
-    return toplevel->screen();
-}
-
-bool EffectWindowImpl::hasOwnShape() const
-{
-    return toplevel->shape();
-}
-
-QSize EffectWindowImpl::size() const
-{
-    return toplevel->size();
-}
-
-QPoint EffectWindowImpl::pos() const
-{
-    return toplevel->pos();
-}
-
-QRect EffectWindowImpl::rect() const
-{
-    return toplevel->rect();
-}
-
-QRect EffectWindowImpl::contentsRect() const
-{
-    return QRect(toplevel->clientPos(), toplevel->clientSize());
-}
-
 QRect EffectWindowImpl::decorationInnerRect() const
 {
     Client *client = dynamic_cast<Client*>(toplevel);
@@ -1523,152 +1478,6 @@ void EffectWindowImpl::deleteProperty(long int atom) const
     deleteWindowProperty(window()->window(), atom);
 }
 
-bool EffectWindowImpl::isMovable() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isMovable();
-    return false;
-}
-
-bool EffectWindowImpl::isMovableAcrossScreens() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isMovableAcrossScreens();
-    return false;
-}
-
-bool EffectWindowImpl::isUserMove() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isMove();
-    return false;
-}
-
-bool EffectWindowImpl::isUserResize() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isResize();
-    return false;
-}
-
-QRect EffectWindowImpl::iconGeometry() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->iconGeometry();
-    return QRect();
-}
-
-bool EffectWindowImpl::isDesktop() const
-{
-    return toplevel->isDesktop();
-}
-
-bool EffectWindowImpl::isDock() const
-{
-    return toplevel->isDock();
-}
-
-bool EffectWindowImpl::isToolbar() const
-{
-    return toplevel->isToolbar();
-}
-
-bool EffectWindowImpl::isMenu() const
-{
-    return toplevel->isMenu();
-}
-
-bool EffectWindowImpl::isNormalWindow() const
-{
-    return toplevel->isNormalWindow();
-}
-
-bool EffectWindowImpl::isSpecialWindow() const
-{
-    if (Client* c = dynamic_cast<Client*>(toplevel))
-        return c->isSpecialWindow();
-    else
-        return true;
-}
-
-bool EffectWindowImpl::isDialog() const
-{
-    return toplevel->isDialog();
-}
-
-bool EffectWindowImpl::isSplash() const
-{
-    return toplevel->isSplash();
-}
-
-bool EffectWindowImpl::isUtility() const
-{
-    return toplevel->isUtility();
-}
-
-bool EffectWindowImpl::isDropdownMenu() const
-{
-    return toplevel->isDropdownMenu();
-}
-
-bool EffectWindowImpl::isPopupMenu() const
-{
-    return toplevel->isPopupMenu();
-}
-
-bool EffectWindowImpl::isTooltip() const
-{
-    return toplevel->isTooltip();
-}
-
-bool EffectWindowImpl::isNotification() const
-{
-    return toplevel->isNotification();
-}
-
-bool EffectWindowImpl::isComboBox() const
-{
-    return toplevel->isComboBox();
-}
-
-bool EffectWindowImpl::isDNDIcon() const
-{
-    return toplevel->isDNDIcon();
-}
-
-NET::WindowType EffectWindowImpl::windowType() const
-{
-    return toplevel->windowType();
-}
-
-bool EffectWindowImpl::isManaged() const
-{
-    return dynamic_cast< const Client* >(toplevel) != NULL;
-}
-
-bool EffectWindowImpl::acceptsFocus() const
-{
-    const Client* client = dynamic_cast< const Client* >(toplevel);
-    if (!client)
-        return true; // We don't actually know...
-    return client->wantsInput();
-}
-
-bool EffectWindowImpl::keepAbove() const
-{
-    const Client* client = dynamic_cast< const Client* >(toplevel);
-    if (!client)
-        return true;
-    return client->keepAbove();
-}
-
-bool EffectWindowImpl::isModal() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isModal();
-    return false;
-}
-
 EffectWindow* EffectWindowImpl::findModal()
 {
     if (Client* c = dynamic_cast< Client* >(toplevel)) {
@@ -1690,49 +1499,11 @@ EffectWindowList EffectWindowImpl::mainWindows() const
     return EffectWindowList();
 }
 
-bool EffectWindowImpl::isSkipSwitcher() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->skipSwitcher();
-    return false;
-}
-
 WindowQuadList EffectWindowImpl::buildQuads(bool force) const
 {
     return sceneWindow()->buildQuads(force);
 }
 
-void EffectWindowImpl::minimize() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel)) {
-        c->minimize();
-    }
-}
-
-void EffectWindowImpl::unminimize() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel)) {
-        c->unminimize();
-    }
-}
-
-void EffectWindowImpl::closeWindow() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel)) {
-        c->closeWindow();
-    }
-}
-
-bool EffectWindowImpl::visibleInClientGroup() const
-{
-    if (Client* c = dynamic_cast< Client* >(toplevel)) {
-        if (!c->clientGroup())
-            return true;
-        return c == c->clientGroup()->visible();
-    }
-    return false;
-}
-
 void EffectWindowImpl::setData(int role, const QVariant &data)
 {
     if (!data.isNull())
