@@ -131,7 +131,7 @@ public:
     };
 
 public slots:
-    void setActive(bool active, bool closingTab = false);   // HACK: closingTab shouldn't be needed
+    void setActive(bool active);
     void toggleActive()  {
         m_mode = ModeCurrentDesktop;
         setActive(!m_activated);
@@ -148,17 +148,12 @@ public slots:
     void globalShortcutChangedAll(const QKeySequence& seq);
     void globalShortcutChangedClass(const QKeySequence& seq);
     // EffectsHandler
-    void slotWindowAdded(EffectWindow *w);
-    void slotWindowClosed(EffectWindow *w);
-    void slotWindowDeleted(EffectWindow *w);
-    void slotWindowGeometryShapeChanged(EffectWindow *w, const QRect &old);
-    // Tab box
-    void slotTabBoxAdded(int mode);
-    void slotTabBoxClosed();
-    void slotTabBoxUpdated();
-    void slotTabBoxKeyEvent(QKeyEvent* event);
+    void slotWindowAdded(KWin::EffectWindow *w);
+    void slotWindowClosed(KWin::EffectWindow *w);
+    void slotWindowDeleted(KWin::EffectWindow *w);
+    void slotWindowGeometryShapeChanged(KWin::EffectWindow *w, const QRect &old);
     // atoms
-    void slotPropertyNotify(EffectWindow* w, long atom);
+    void slotPropertyNotify(KWin::EffectWindow* w, long atom);
 
 private slots:
     void closeWindow();
@@ -210,12 +205,11 @@ private:
     // User configuration settings
     QList<ElectricBorder> m_borderActivate;
     QList<ElectricBorder> m_borderActivateAll;
+    QList<ElectricBorder> m_borderActivateClass;
     int m_layoutMode;
     bool m_showCaptions;
     bool m_showIcons;
     bool m_doNotCloseWindows;
-    bool m_tabBoxAllowed;
-    bool m_tabBoxAlternativeAllowed;
     int m_accuracy;
     bool m_fillGaps;
     double m_fadeDuration;
@@ -227,7 +221,6 @@ private:
     double m_decalOpacity;
     Window m_input;
     bool m_hasKeyboardGrab;
-    bool m_tabBoxEnabled;
     PresentWindowsMode m_mode;
     int m_desktop;
     EffectWindowList m_selectedWindows;
