@@ -185,9 +185,6 @@ void StartupFeedbackEffect::paintScreen(int mask, QRegion region, ScreenPaintDat
         default:
             return; // safety
         }
-#ifndef KWIN_HAVE_OPENGLES
-        glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
-#endif
         glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         texture->bind();
@@ -236,9 +233,6 @@ void StartupFeedbackEffect::paintScreen(int mask, QRegion region, ScreenPaintDat
         }
         texture->unbind();
         glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-        glPopAttrib();
-#endif
     }
 }
 
