@@ -165,12 +165,12 @@ KWIN_EFFECT_SUPPORTED(wobblywindows, WobblyWindowsEffect::supported())
 WobblyWindowsEffect::WobblyWindowsEffect()
 {
     reconfigure(ReconfigureAll);
-    connect(effects, SIGNAL(windowAdded(EffectWindow*)), this, SLOT(slotWindowAdded(EffectWindow*)));
-    connect(effects, SIGNAL(windowClosed(EffectWindow*)), this, SLOT(slotWindowClosed(EffectWindow*)));
-    connect(effects, SIGNAL(windowStartUserMovedResized(EffectWindow*)), this, SLOT(slotWindowStartUserMovedResized(EffectWindow*)));
-    connect(effects, SIGNAL(windowStepUserMovedResized(EffectWindow*,QRect)), this, SLOT(slotWindowStepUserMovedResized(EffectWindow*,QRect)));
-    connect(effects, SIGNAL(windowFinishUserMovedResized(EffectWindow*)), this, SLOT(slotWindowFinishUserMovedResized(EffectWindow*)));
-    connect(effects, SIGNAL(windowMaximizedStateChanged(EffectWindow*,bool,bool)), this, SLOT(slotWindowMaximizeStateChanged(EffectWindow*,bool,bool)));
+    connect(effects, SIGNAL(windowAdded(KWin::EffectWindow*)), this, SLOT(slotWindowAdded(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(windowClosed(KWin::EffectWindow*)), this, SLOT(slotWindowClosed(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(windowStartUserMovedResized(KWin::EffectWindow*)), this, SLOT(slotWindowStartUserMovedResized(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(windowStepUserMovedResized(KWin::EffectWindow*,QRect)), this, SLOT(slotWindowStepUserMovedResized(KWin::EffectWindow*,QRect)));
+    connect(effects, SIGNAL(windowFinishUserMovedResized(KWin::EffectWindow*)), this, SLOT(slotWindowFinishUserMovedResized(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(windowMaximizedStateChanged(KWin::EffectWindow*,bool,bool)), this, SLOT(slotWindowMaximizeStateChanged(KWin::EffectWindow*,bool,bool)));
 }
 
 WobblyWindowsEffect::~WobblyWindowsEffect()
@@ -291,8 +291,6 @@ void WobblyWindowsEffect::prePaintScreen(ScreenPrePaintData& data, int time)
     // screen won't be repainted, resulting in artefacts.
     // Could we just set a subset of the screen to be repainted ?
     if (windows.count() != 0) {
-        data.mask |= PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_WITHOUT_FULL_REPAINTS;
-
         m_updateRegion = QRegion();
     }
 
@@ -374,6 +372,7 @@ void WobblyWindowsEffect::slotWindowStartUserMovedResized(EffectWindow *w)
 
 void WobblyWindowsEffect::slotWindowStepUserMovedResized(EffectWindow *w, const QRect &geometry)
 {
+    Q_UNUSED(geometry)
     if (windows.contains(w)) {
         WindowWobblyInfos& wwi = windows[w];
         QRect rect = w->geometry();
@@ -399,7 +398,9 @@ void WobblyWindowsEffect::slotWindowFinishUserMovedResized(EffectWindow *w)
 
 void WobblyWindowsEffect::slotWindowMaximizeStateChanged(EffectWindow *w, bool horizontal, bool vertical)
 {
-    if (!m_moveEffectEnabled || w->isSpecialWindow())
+    Q_UNUSED(horizontal)
+    Q_UNUSED(vertical)
+    if (w->isUserMove() || !m_moveEffectEnabled || w->isSpecialWindow())
         return;
 
     if (m_moveWobble && m_resizeWobble) {
