@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "lanczosfilter.h"
 #include "effects.h"
+#include "options.h"
 
 #include <kwinglutils.h>
 #include <kwinglplatform.h>
@@ -59,14 +60,12 @@ void LanczosFilter::init()
         kWarning(1212) << "Lanczos Filter forced on by environment variable";
     }
 
-    KSharedConfigPtr config = KSharedConfig::openConfig("kwinrc");
-
-    if (!force && config->group("Compositing").readEntry("GLTextureFilter", 2) != 2)
+    if (!force && options->glSmoothScale() != 2)
         return; // disabled by config
 
     // The lanczos filter is reported to be broken with the Intel driver and Mesa 7.10
     GLPlatform *gl = GLPlatform::instance();
-    if (!force && gl->driver() == Driver_Intel && gl->mesaVersion() >= kVersionNumber(7, 10))
+    if (!force && gl->driver() == Driver_Intel && gl->mesaVersion() >= kVersionNumber(7, 10) && gl->chipClass() < SandyBridge)
         return;
     // With fglrx the ARB Shader crashes KWin (see Bug #270818 and #286795) and GLSL Shaders are not functional
     if (!force && gl->driver() == Driver_Catalyst) {
@@ -192,6 +191,7 @@ void LanczosFilter::performPaint(EffectWindowImpl* w, int mask, QRegion region,
             int tw = width * data.xScale;
             int th = height * data.yScale;
             const QRect textureRect(tx, ty, tw, th);
+            const bool hardwareClipping = !(QRegion(textureRect)-region).isEmpty();
 
             int sw = width;
             int sh = height;
@@ -200,6 +200,9 @@ void LanczosFilter::performPaint(EffectWindowImpl* w, int mask, QRegion region,
             if (cachedTexture) {
                 if (cachedTexture->width() == tw && cachedTexture->height() == th) {
                     cachedTexture->bind();
+                    if (hardwareClipping) {
+                        glEnable(GL_SCISSOR_TEST);
+                    }
                     if (ShaderManager::instance()->isValid()) {
                         glEnable(GL_BLEND);
                         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -213,15 +216,18 @@ void LanczosFilter::performPaint(EffectWindowImpl* w, int mask, QRegion region,
                         shader->setUniform(GLShader::Saturation, data.saturation);
                         shader->setUniform(GLShader::AlphaToOne, 0);
 
-                        cachedTexture->render(textureRect, textureRect);
+                        cachedTexture->render(region, textureRect, hardwareClipping);
 
                         ShaderManager::instance()->popShader();
                         glDisable(GL_BLEND);
                     } else {
                         prepareRenderStates(cachedTexture, data.opacity, data.brightness, data.saturation);
-                        cachedTexture->render(textureRect, textureRect);
+                        cachedTexture->render(region, textureRect, hardwareClipping);
                         restoreRenderStates(cachedTexture, data.opacity, data.brightness, data.saturation);
                     }
+                    if (hardwareClipping) {
+                        glDisable(GL_SCISSOR_TEST);
+                    }
                     cachedTexture->unbind();
                     m_timer.start(5000, this);
                     return;
@@ -331,6 +337,9 @@ void LanczosFilter::performPaint(EffectWindowImpl* w, int mask, QRegion region,
             glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, m_offscreenTex->height() - th, tw, th);
             GLRenderTarget::popRenderTarget();
 
+            if (hardwareClipping) {
+                glEnable(GL_SCISSOR_TEST);
+            }
             if (ShaderManager::instance()->isValid()) {
                 glEnable(GL_BLEND);
                 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -344,15 +353,18 @@ void LanczosFilter::performPaint(EffectWindowImpl* w, int mask, QRegion region,
                 shader->setUniform(GLShader::Saturation, data.saturation);
                 shader->setUniform(GLShader::AlphaToOne, 0);
 
-                cache->render(textureRect, textureRect);
+                cache->render(region, textureRect, hardwareClipping);
 
                 ShaderManager::instance()->popShader();
                 glDisable(GL_BLEND);
             } else {
                 prepareRenderStates(cache, data.opacity, data.brightness, data.saturation);
-                cache->render(textureRect, textureRect);
+                cache->render(region, textureRect, hardwareClipping);
                 restoreRenderStates(cache, data.opacity, data.brightness, data.saturation);
             }
+            if (hardwareClipping) {
+                glDisable(GL_SCISSOR_TEST);
+            }
 
             cache->unbind();
             w->setData(LanczosCacheRole, QVariant::fromValue(static_cast<void*>(cache)));
@@ -388,7 +400,12 @@ void LanczosFilter::timerEvent(QTimerEvent *event)
 
 void LanczosFilter::prepareRenderStates(GLTexture* tex, double opacity, double brightness, double saturation)
 {
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGLES
+    Q_UNUSED(tex)
+    Q_UNUSED(opacity)
+    Q_UNUSED(brightness)
+    Q_UNUSED(saturation)
+#else
     const bool alpha = true;
     // setup blending of transparent windows
     glPushAttrib(GL_ENABLE_BIT);
@@ -500,7 +517,12 @@ void LanczosFilter::prepareRenderStates(GLTexture* tex, double opacity, double b
 
 void LanczosFilter::restoreRenderStates(GLTexture* tex, double opacity, double brightness, double saturation)
 {
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGLES
+    Q_UNUSED(tex)
+    Q_UNUSED(opacity)
+    Q_UNUSED(brightness)
+    Q_UNUSED(saturation)
+#else
     if (opacity != 1.0 || saturation != 1.0 || brightness != 1.0f) {
         if (saturation != 1.0 && tex->saturationSupported()) {
             glActiveTexture(GL_TEXTURE3);
@@ -525,6 +547,9 @@ void LanczosFilter::restoreRenderStates(GLTexture* tex, double opacity, double b
 LanczosShader::LanczosShader(QObject* parent)
     : QObject(parent)
     , m_shader(0)
+    , m_uTexUnit(0)
+    , m_uOffsets(0)
+    , m_uKernel(0)
     , m_arbProgram(0)
 {
 }
@@ -560,7 +585,7 @@ void LanczosShader::unbind()
     else {
         int boundObject;
         glGetProgramivARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_BINDING_ARB, &boundObject);
-        if (boundObject == m_arbProgram) {
+        if (boundObject == (int)m_arbProgram) {
             glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, 0);
             glDisable(GL_FRAGMENT_PROGRAM_ARB);
         }
