@@ -455,6 +455,7 @@ void PlasmaApp::showInteractiveConsole()
     if (!console) {
         m_console = console = new InteractiveConsole(m_corona);
     }
+    m_console.data()->setMode(InteractiveConsole::PlasmaConsole);
 
     KWindowSystem::setOnDesktop(console->winId(), KWindowSystem::currentDesktop());
     console->show();
@@ -470,6 +471,22 @@ void PlasmaApp::loadScriptInInteractiveConsole(const QString &script)
     }
 }
 
+void PlasmaApp::showInteractiveKWinConsole()
+{
+    showInteractiveConsole();
+    if (m_console) {
+        m_console.data()->setMode(InteractiveConsole::KWinConsole);
+    }
+}
+
+void PlasmaApp::loadKWinScriptInInteractiveConsole (const QString &script)
+{
+    showInteractiveKWinConsole();
+    if (m_console) {
+        m_console.data()->loadScript(script);
+    }
+}
+
 void PlasmaApp::panelHidden(bool hidden)
 {
     if (hidden) {
