@@ -379,16 +379,16 @@ void PowerDevilHALBackend::slotButtonPressed(Solid::Button::ButtonType type)
 
     switch(type) {
     case Solid::Button::PowerButton:
-        setButtonPressed(PowerButton);
+        emit buttonPressed(PowerButton);
         break;
     case Solid::Button::SleepButton:
-        setButtonPressed(SleepButton);
+        emit buttonPressed(SleepButton);
         break;
     case Solid::Button::LidButton:
         if (button->stateValue()) {
-            setButtonPressed(LidClose);
+            emit buttonPressed(LidClose);
         } else {
-            setButtonPressed(LidOpen);
+            emit buttonPressed(LidOpen);
         }
         break;
     default:
